@@ -2,6 +2,7 @@ import argparse
 import csv
 import io
 import logging
+import shutil
 import threading
 import time
 import tkinter as tk
@@ -30,6 +31,31 @@ class ArchiveBase:
     prev_cache = 2
     next_cache = 10
 
+    support_image_type = [
+        ".bmp",
+        ".dib",
+        ".eps",
+        ".gif",
+        ".icns",
+        ".ico",
+        ".im",
+        ".jpg",
+        ".jpeg",
+        ".msp",
+        ".pcx",
+        ".png",
+        ".ppm",
+        ".sgi",
+        ".spider",
+        ".tga",
+        ".tiff",
+        ".webv",
+        ".xbm",
+        "svg",
+    ]
+    support_archive_type = [".zip", ".rar", ".7z", ".pdf"]
+    support_type = support_image_type + support_archive_type
+
     def __init__(self, multi_read=False):
         self.file_path = None
         self.data = None
@@ -46,6 +72,13 @@ class ArchiveBase:
     def __del__(self):
         self.close()
 
+    def filtering_file_list(self):
+        self.file_list = [
+            f
+            for f in self.file_list
+            if str(f)[-1] != "/" and Path(f).suffix.lower() in self.support_type
+        ]
+
     def close(self):
         self.stop = True
         if self.data is not None:
@@ -177,6 +210,7 @@ class DirectoryArchive(ArchiveBase):
         self.file_list = natsorted(
             list(Path(self.file_path.parent).glob("*")), key=lambda x: str(x)
         )
+        self.filtering_file_list()
         logger.debug(self.file_list)
         self.i = self.file_list.index(Path(file_path))
         logger.debug(f"self.i = {self.i}")
@@ -191,7 +225,6 @@ class DirectoryArchive(ArchiveBase):
     def getitem(self, i):
         logger.debug(f"i = {i}")
         if 0 <= i < len(self):
-            self.file_path = self.file_list[i]
             return self.file_list[i], None
         else:
             return "", None
@@ -220,7 +253,7 @@ class ZipArchive(ArchiveBase):
             self.file_list = natsorted(f.namelist())
 
         logger.debug("to list")
-        self.file_list = [f for f in self.file_list if f[-1] != "/"]
+        self.filtering_file_list()
         logger.debug(self.file_list)
         logger.debug("return")
 
@@ -268,7 +301,7 @@ class RarArchive(ArchiveBase):
             self.file_list = natsorted(f.namelist())
 
         logger.debug("open rar")
-        self.file_list = [f for f in self.file_list if f[-1] != "/"]
+        self.filtering_file_list()
         logger.debug(self.file_list)
 
     def getitem(self, i):
@@ -311,7 +344,7 @@ class SevenZipArchive(ArchiveBase):
         with py7zr.SevenZipFile(fp, mode="r") as f:
             self.file_list = natsorted(f.getnames())
 
-        self.file_list = [f for f in self.file_list if f[-1] != "/"]
+        self.filtering_file_list()
         logger.debug(self.file_list)
         logger.debug("return")
 
@@ -693,6 +726,80 @@ class ImageFrame(tk.Canvas):
         return self.resize(image, size, algo)
 
 
+class MoveFile:
+    def __init__(self):
+        self.ret = False
+        pass
+
+    def move_file(self, move_to_list, file_path):
+        self.file_path = file_path
+        self.move_to_list = move_to_list
+        if len(self.move_to_list) == 0:
+            messagebox.showwarning("No place is registered", "No place is registered")
+            return
+
+        self.child = tk.Toplevel()
+        self.child.attributes("-type", "dialog")
+        # self.child.focus_set()
+        self.child.focus_force()
+        self.child.grab_set()
+
+        self.child.bind("<KeyPress>", self._move)
+        ttk.Label(self.child, text=f"Move {file_path} to").pack(
+            side="top", expand=True, fill="x"
+        )
+
+        for k, v in self.move_to_list.items():
+            ttk.Label(self.child, text=f"{k}:{v}").pack(
+                side="top", expand=True, fill="x"
+            )
+
+        ttk.Label(self.child, text="q, Esc, Ctrl+[, [: Quit").pack(
+            side="top", expand=True, fill="x"
+        )
+
+        self.child.bind("<Escape>", lambda event: self.child.destroy())
+        self.child.bind("[", lambda event: self.child.destroy())
+        self.child.bind("q", lambda event: self.child.destroy())
+        self.child.wait_window()
+        return self.ret
+
+    def _move(self, event):
+        self.child.destroy()
+        file_path = self.file_path
+        key = event.keysym
+
+        to = self.move_to_list.get(key)
+        if to is None:
+            messagebox.showwarning(
+                "Such place is not in list.", f"Such place is not in list. {key}"
+            )
+            self.ret = False
+            return
+
+        to = Path(to)
+
+        if not to.exists():
+            messagebox.showwarning(
+                "Such directory does not exist.", f"Such directory does not exist. {to}"
+            )
+            self.ret = False
+            return
+
+        to = to / file_path.name
+        if to.exists() and not messagebox.askokcancel(
+            "File exists.", "File exists. Overwrite?"
+        ):
+            logger.debug("Do not overwrite.")
+            self.ret = False
+            return
+
+        logger.debug(f"Move {file_path} -> {to}")
+
+        shutil.move(file_path, to)
+        self.ret = True
+
+
 class Config:
     default_config = """\
 [Setting]
@@ -727,7 +834,7 @@ DownScale   = Lanczos
 [Keymap]
 
 DoublePage  = d
-TrashFile   = Delete
+
 
 # You can use repetition for NextPage and PrevPage.
 # For example, 2h means goto next 2 page, type 100h go to next 100 page.
@@ -738,6 +845,10 @@ PrevPage    = l
 NextArchive = j
 PrevArchive = k
 
+Head        = g
+Tail        = G
+
+
 FitNone     = N
 FitWidth    = W
 FitHeight   = H
@@ -745,26 +856,41 @@ FitBoth     = B
 
 PageOrder   = o
 
-Head        = g
-Tail        = G
+
+TrashFile   = Delete
+MoveFile    = m
 
 Quit        = q
 FullScreen  = f
 Reload      = r
+
+
+[MoveToList]
+
+# When you press MoveFile key, then press key registered.
+# File will be moved to registered place.
+
+# a = /home/GuiltyCat/images/fantastic
+# b = /home/GuiltyCat/images/bravo
+# b = /home/GuiltyCat/images/wonderful
+#
 """
 
     def __init__(self):
         self.keymap = {}
         self.setting = {}
+        self.move_to_list = {}
         pass
 
     def open(self, file_path):
         file_path = Path(file_path)
         fp = file_path
         if file_path.exists():
+            logger.debug("config file exists. reading")
             with open(fp, mode="r", newline="") as f:
                 self._load(f)
         else:
+            logger.debug("config file do not exists. read default.")
             with io.StringIO(self.default_config) as f:
                 self._load(f)
 
@@ -784,6 +910,9 @@ Reload      = r
             if row[0] == "[Keymap]":
                 config = self.keymap
                 continue
+            if row[0] == "[MoveToList]":
+                config = self.move_to_list
+                continue
 
             if config is None:
                 continue
@@ -830,6 +959,7 @@ class SaltViewer(tk.Tk):
             "FitHeight": self.fit_height,
             "FitBoth": self.fit_both,
             "FitNone": self.fit_none,
+            "MoveFile": self.move_file,
             "Quit": self.quit,
             "Reload": self.reload,
             "FullScreen": self.full_screen,
@@ -854,6 +984,37 @@ class SaltViewer(tk.Tk):
 
         self.load_config(args)
 
+    def move_file(self, event):
+
+        logger.debug("called")
+
+        file_path = self.archive.file_path
+        move_to_list = self.config.move_to_list
+        logger.debug(f"{file_path}, {move_to_list}")
+
+        archive = DirectoryArchive(file_path)
+        if len(archive) == 0:
+            self.archive.close()
+            archive.close()
+            self.quit(None)
+
+            if not MoveFile().move_file(move_to_list, file_path):
+                logger.debug("move failed")
+                return
+            return
+
+        next_file_path = archive.next()[0]
+        if next_file_path == file_path:
+            next_file_path = archive.prev()[0]
+
+        archive.close()
+        if not MoveFile().move_file(move_to_list, file_path):
+            logger.debug("move failed")
+            return
+        self.archive.close()
+        self.open(next_file_path)
+        pass
+
     def reload(self, event):
         self.archive.cache = {}
         self.current_page()
@@ -882,6 +1043,8 @@ class SaltViewer(tk.Tk):
 
         logger.debug("overwrite settings")
         for k, v in args.items():
+            if v is None:
+                continue
             self.config.setting[k] = v
 
         for name, key in self.config.keymap.items():
@@ -907,6 +1070,7 @@ class SaltViewer(tk.Tk):
             elif name == "DownScale":
                 self.image.select_down_scale_algorithm(key)
             elif name == "DefaultFullScreen":
+                logger.debug("DefaultFullScreen")
                 self.attributes("-fullscreen", key == "True")
             elif name == "DefaultPrevCache":
                 ArchiveBase.prev_cache = int(key)
@@ -999,7 +1163,7 @@ class SaltViewer(tk.Tk):
             next_file_path = archive.next()[0]
             if next_file_path == file_path:
                 next_file_path = archive.prev()[0]
-            archive.close() 
+            archive.close()
             self.archive.trash()
             self.archive.close()
             self.open(next_file_path)
@@ -1100,6 +1264,7 @@ class SaltViewer(tk.Tk):
             return DirectoryArchive(file_path, data)
 
     def open_file(self, file_path, data=None):
+        file_path = Path(file_path)
         logger.debug("called")
 
         logger.debug("set title")
@@ -1111,7 +1276,6 @@ class SaltViewer(tk.Tk):
         self.title(page + title)
         self.image.title = title
 
-        file_path = Path(file_path)
         logger.debug(file_path)
         suffix = file_path.suffix.lower()
         logger.debug(suffix)
@@ -1485,7 +1649,9 @@ def main():
 
     default_config_path = str(Path.home() / ".svrc")
 
-    parser = argparse.ArgumentParser(description="SaltViewer")
+    parser = argparse.ArgumentParser(
+        description="SaltViewer. Simple (archived) image viewer (https://github.com/GuiltyCat/SaltViewer)"
+    )
     parser.add_argument(
         "path", help="image file or archive file", type=str, default=None
     )
@@ -1505,44 +1671,45 @@ def main():
         "--debug", help="run as debug mode. All log is printed.", action="store_true"
     )
     parser.add_argument(
-        "--fullscreen", help="run as fullscreen mode", action="store_true"
+        "--fullscreen", help="run as fullscreen mode", action="store_true", default=None
     )
     parser.add_argument(
         "--prev_cache",
-        help="number of previous page cache. Default is %(default)s",
+        help="number of previous page cache. Default is 4",
         type=int,
-        default=4,
+        default=None,
     )
     parser.add_argument(
         "--next_cache",
-        help="number of previous page cache. Default is %(default)s",
+        help="number of previous page cache. Default is 10",
         type=int,
-        default=10,
+        default=None,
     )
     parser.add_argument(
         "--fit_mode",
-        help="fit_mode. Both, Raw, Width, Height.  Default is %(default)s",
-        default="Both",
+        help="fit_mode. Both, Raw, Width, Height.  Default is Both",
+        default=None,
     )
     parser.add_argument(
         "--page_order",
-        help="page order in double page mode. right2left or left2right. Default is %(default)s",
-        default="right2left",
+        help="page order in double page mode. right2left or left2right. Default is right2left",
+        default=None,
     )
     parser.add_argument(
         "--double",
         help="Double page mode. Default is %(default)s.",
         action="store_true",
+        default=None,
     )
     parser.add_argument(
         "--upscale",
-        help="Upscale algorithm. Nearest, Box, Bilinear, Hamming, Bicubic, Lanczos. Default is %(default)s.",
-        default="Lanczos",
+        help="Upscale algorithm. Nearest, Box, Bilinear, Hamming, Bicubic, Lanczos. Default is Lanczos",
+        default=None,
     )
     parser.add_argument(
         "--downscale",
-        help="Downscale algorithm. Nearest, Box, Bilinear, Hamming, Bicubic, Lanczos. Default is %(default)s.",
-        default="Lanczos",
+        help="Downscale algorithm. Nearest, Box, Bilinear, Hamming, Bicubic, Lanczos. Default is Lanczos",
+        default=None,
     )
 
     args = parser.parse_args()
@@ -1556,7 +1723,8 @@ def main():
 
     if args.default_config:
         logger.debug("write default config")
-        Config.write_default_config(args.path)
+        Config().write_default_config(args.path)
+        return
 
     if args.debug:
         logger.debug("setLevel DEBUG")
